@@ -11,7 +11,7 @@ from . import prehash
 from . import queryprovider
 from .query import query_ns_records
 from . import rrfile
-from .exception import N3MapError, FileParseError
+from .exception import N3MapError, FileParseError, HashLimitReached
 from .nsec3walker import NSEC3Walker
 from .predict import create_zone_predictor
 from .nsecwalker import NSECWalkerN, NSECWalkerMixed, NSECWalkerA
@@ -19,10 +19,6 @@ from .nsecwalker import NSECWalkerN, NSECWalkerMixed, NSECWalkerA
 import n3map.name
 import n3map.walker
 
-def _compute_query_interval(n, unit):
-    units = { 's': 1.0, 'm' : 60.0, 'h': 3600.0 }
-    return units[unit]/n
-
 def _def_num_of_processes():
     try:
         ncpus = multiprocessing.cpu_count()
@@ -34,6 +30,10 @@ def _def_num_of_processes():
         return ncpus - 1
     return 1
 
+def _compute_query_interval(n, unit):
+    units = { 's': 1.0, 'm' : 60.0, 'h': 3600.0 }
+    return units[unit]/n
+
 def _query_interval(s):
     p = re.compile('^(([0-9]\.|[1-9][0-9]*[.]?)[0-9]*)/([smh])$')
     m = p.match(s)
@@ -45,6 +45,25 @@ def _query_interval(s):
         raise ValueError
 
 
+def _human_number(s):
+    units = {
+            'K': 1000,
+            'M': 1000000,
+            'G': 1000000000,
+            'T': 1000000000000,
+            }
+    m = re.fullmatch(r'(\d+)([a-zA-Z]?)', s)
+    if m is None:
+        raise ValueError
+    n = int(m.group(1))
+    unit = m.group(2)
+    if unit != '':
+        try:
+            n *= units[unit.upper()]
+        except KeyError:
+            raise ValueError
+    return n
+
 def check_part_of_zone(rr, zone):
     if not rr.part_of_zone(zone):
         raise N3MapError(("not all read records are part of the specified zone"))
@@ -54,7 +73,8 @@ def get_nameservers(zone, ipproto='', ns_names=None):
         return queryprovider.nameserver_from_text(ipproto, *ns_names)
 
     ns_names = query_ns_records(zone)
-    nslist = queryprovider.nameserver_from_text(ipproto, *ns_names)
+    nslist = queryprovider.nameserver_from_text(ipproto, *ns_names,
+                                                ignore_unresolved=True)
     for ns in nslist:
         log.info("using nameserver: ", str(ns))
     return nslist
@@ -186,7 +206,9 @@ def n3map_main(argv):
                                  output_file=output_rrfile,
                                  stats=stats,
                                  predictor=predictor,
-                                 aggressive=options['aggressive'])
+                                 aggressive=options['aggressive'],
+                                 hashlimit=options['hashlimit']
+                                 )
 
         elif options['zone_type'] == 'nsec':
             if output_rrfile is not None:
@@ -221,9 +243,18 @@ def n3map_main(argv):
         finished = False
         if walker is not None:
             starttime = time.monotonic()
-            walker.walk()
+            stopped_prematurely = False
+            try:
+                walker.walk()
+            except HashLimitReached:
+                stopped_prematurely = True
             elapsed = timedelta(seconds=time.monotonic() - starttime)
-            log.info("finished mapping of {0:s} in {1:s}".format( str(zone), str(elapsed)))
+            if stopped_prematurely:
+                log.info("stopped mapping of {0:s} after {1:s}: hashlimit reached"
+                         .format( str(zone), str(elapsed)))
+            else:
+                log.info("finished mapping of {0:s} in {1:s}"
+                         .format( str(zone), str(elapsed)))
             finished = True
 
         if output_rrfile is not None:
@@ -253,6 +284,7 @@ def default_options():
             'start' : None,
             'end' : None,
             'label_counter' : None,
+            'hashlimit' : 0,
             'timeout' : 2500,
             'max_retries' : 5,
             'max_errors' : 1,
@@ -283,6 +315,7 @@ def parse_arguments(argv):
             'ignore-overlapping',
             'input=',
             'label-counter=',
+            'hashlimit=',
             'ldh',
             'limit-rate=',
             'max-retries=',
@@ -352,6 +385,12 @@ def parse_arguments(argv):
             if options['label_counter']  < 0:
                 invalid_argument(opt, arg)
 
+        elif opt in ('--hashlimit',):
+            try:
+                options['hashlimit'] = _human_number(arg)
+            except ValueError:
+                invalid_argument(opt, arg)
+
         elif opt in ('--ignore-overlapping',):
             options['ignore_overlapping'] = True
 
@@ -556,6 +595,9 @@ NSEC3 Options:
       --processes=N          defines the number of pre-hashing processes.
                                Default is 1 or the number of CPUs - 1 on
                                multiprocessor systems ({processes:d} on this system)
+      --hashlimit=N[K|M|G|T]
+                             stop the enumeration after checking N hashes, even
+                               if it is not finished. Default = 0 (unlimited).
 
 Advanced NSEC3 Options:
   Use with caution.
